interface Deferred<T> extends Promise<T> {
    readonly state: "pending" | "fulfilled" | "rejected";
    resolve(value?: T | PromiseLike<T>): void;
    reject(reason?: any): void;
}

declare class Queue<T> {
    #private;
    constructor(limit?: number);
    [Symbol.asyncIterator](): AsyncIterableIterator<T>;
    pop(): Deferred<T>;
    push(data: T): void;
}

declare type Percent = `${number}%`;

declare type SceneEventType = "update" | "enter" | "leave";
declare type SceneEvent = {
    target: HTMLElement;
    progress: number;
};
interface SceneConfig {
    play: number | Percent;
    duration: number | Percent;
}
declare const DefaultSceneConfig: SceneConfig;
declare class Scene {
    #private;
    get emitter(): Queue<[SceneEventType, SceneEvent]>;
    constructor(scroller: Scroller, el: HTMLElement, cfg: SceneConfig);
    update(): void;
}

declare type ScrollDirection = "forward" | "reverse" | "paused";
declare class Scroller {
    #private;
    scrollY: number;
    direction: ScrollDirection;
    get clientHeight(): number;
    destory: () => void;
    constructor(el?: HTMLElement);
    addScene(el: HTMLElement, cfg: SceneConfig): Scene;
    addSceneWithDefault(el: HTMLElement): Scene;
}

export { DefaultSceneConfig, Scene, Scroller };
